/*
 * Copyright (C) 2024
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef P1_H
#define P1_H

// -- Includes: -----------------------------------------------------------
#include <cstdint>

// -- Using: --------------------------------------------------------------
using namespace std;

// -- Typdefs: ------------------------------------------------------------
/** El tipo de los elementos de la matriz de bajo nivel. */
using etype_t  = int32_t;
/// El tipo de la fila de la matriz de bajo nivel
using row_t    = etype_t*;
//! El tipo matriz de bajo nivel.
using matrix_t = row_t*;

/** Tipo matriz de alto nivel.
 * Conoce las dimensiones de una matriz de bajo nivel.
 */
struct Matrix {
  matrix_t data;
  uint32_t rows;
  uint32_t cols;
};

// Matriz nula o matriz vacia
const Matrix null_matrix = {nullptr, 0, 0};

// -- Ejercicio 1: --------------------------------------------------------

/// Muestra la matriz por pantalla
void mshow(const Matrix m);
/// Reserva la memoria para la matriz y la inicializa al valor v
Matrix  mreserve(uint32_t r, uint32_t c, etype_t v = 0);
/// Libera la memoria de la matriz
void mfree(Matrix& m);

/// Tipo que representa un puntero a función
using fp_t = etype_t(*)(etype_t);

/// Modifica cada elemento[i][j] de la matriz con el resultado de
/// f(m[i][j]).
/// \param m La matriz con la que se trabaja
/// \param f La función que recibe como argumento cada elemento la
/// matriz m y realiza una operacion con el.
void map_elements(Matrix m, fp_t f);

// -- Ejercicio 2: --------------------------------------------------------

/// Crea una nueva matriz con 'u' unidades mas que la original en la
/// que las nuevas celdas se inicializan al valor 'v'.
Matrix matrix_grow(const Matrix m, uint32_t u, etype_t v = 0);
/// Crea una nueva matriz con 'u' unidades menos que la original.
Matrix matrix_shrink(const Matrix m, uint32_t u);

#endif /* P1_H */
